@@ -119,7 +119,7 @@ static guint arrow_button_signals[LAST_SIGNAL];
 
 
 
-G_DEFINE_TYPE (XfceArrowButton, xfce_arrow_button, GTK_TYPE_TOGGLE_BUTTON)
+G_DEFINE_TYPE_WITH_PRIVATE (XfceArrowButton, xfce_arrow_button, GTK_TYPE_TOGGLE_BUTTON)
 
 
 
@@ -129,8 +129,6 @@ xfce_arrow_button_class_init (XfceArrowButtonClass * klass)
   GObjectClass   *gobject_class;
   GtkWidgetClass *gtkwidget_class;
 
-  g_type_class_add_private (klass, sizeof (XfceArrowButtonPrivate));
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->get_property = xfce_arrow_button_get_property;
   gobject_class->set_property = xfce_arrow_button_set_property;
@@ -191,7 +189,7 @@ xfce_arrow_button_init (XfceArrowButton *button)
   GtkCssProvider  *provider;
 #endif
 
-  button->priv = G_TYPE_INSTANCE_GET_PRIVATE (button, XFCE_TYPE_ARROW_BUTTON, XfceArrowButtonPrivate);
+  button->priv = xfce_arrow_button_get_instance_private (button);
 
   /* initialize button values */
   button->priv->arrow_type = GTK_ARROW_UP;
@@ -364,6 +362,7 @@ xfce_arrow_button_get_preferred_width (GtkWidget *widget,
   GtkStyleContext *context;
   GtkBorder        padding, border;
 
+  minimum_child_width = natural_child_width = 0;
   child = gtk_bin_get_child (GTK_BIN (widget));
   if (child != NULL
       && gtk_widget_get_visible (child))
@@ -389,8 +388,7 @@ xfce_arrow_button_get_preferred_width (GtkWidget *widget,
       context = gtk_widget_get_style_context (widget);
       gtk_style_context_get_padding (context, gtk_widget_get_state_flags (widget), &padding);
       gtk_style_context_get_border (context, gtk_widget_get_state_flags (widget), &border);
-      natural_child_width = (ARROW_WIDTH + padding.left + padding.right + border.left + border.right);
-      minimum_child_width = natural_child_width - ARROW_WIDTH;
+      minimum_child_width = natural_child_width = (ARROW_WIDTH + padding.left + padding.right + border.left + border.right);
     }
 
   if (minimum_width != NULL)
@@ -413,6 +411,7 @@ xfce_arrow_button_get_preferred_height (GtkWidget *widget,
   GtkStyleContext *context;
   GtkBorder        padding, border;
 
+  minimum_child_height = natural_child_height = 0;
   child = gtk_bin_get_child (GTK_BIN (widget));
   if (child != NULL
       && gtk_widget_get_visible (child))
@@ -438,8 +437,7 @@ xfce_arrow_button_get_preferred_height (GtkWidget *widget,
       context = gtk_widget_get_style_context (widget);
       gtk_style_context_get_padding (context, gtk_widget_get_state_flags (widget), &padding);
       gtk_style_context_get_border (context, gtk_widget_get_state_flags (widget), &border);
-      natural_child_height = (ARROW_WIDTH + padding.top + padding.bottom + border.top + border.bottom);
-      minimum_child_height = natural_child_height - ARROW_WIDTH;
+      minimum_child_height = natural_child_height = (ARROW_WIDTH + padding.top + padding.bottom + border.top + border.bottom);
     }
 
 
@@ -514,6 +512,7 @@ xfce_arrow_button_size_request (GtkWidget      *widget,
   XfceArrowButton *button = XFCE_ARROW_BUTTON (widget);
   GtkWidget *child;
 
+  requisition->height = requisition->width = 0;
   child = gtk_bin_get_child (GTK_BIN (widget));
   if (child != NULL && GTK_WIDGET_VISIBLE (child))
     {
