@@ -93,6 +93,8 @@ static void          xfce_panel_plugin_unregister_menu        (GtkMenu
                                                                XfcePanelPlugin                  *plugin);
 static void          xfce_panel_plugin_set_size               (XfcePanelPluginProvider          *provider,
                                                                gint                              size);
+static void          xfce_panel_plugin_set_icon_size          (XfcePanelPluginProvider          *provider,
+                                                               gint                              icon_size);
 static void          xfce_panel_plugin_set_mode               (XfcePanelPluginProvider          *provider,
                                                                XfcePanelPluginMode               mode);
 static void          xfce_panel_plugin_set_nrows              (XfcePanelPluginProvider          *provider,
@@ -129,6 +131,7 @@ enum
   PROP_UNIQUE_ID,
   PROP_ORIENTATION,
   PROP_SIZE,
+  PROP_ICON_SIZE,
   PROP_SMALL,
   PROP_SCREEN_POSITION,
   PROP_EXPAND,
@@ -175,6 +178,7 @@ struct _XfcePanelPluginPrivate
   gchar               *property_base;
   gchar              **arguments;
   gint                 size; /* single row size */
+  gint                 icon_size;
   guint                expand : 1;
   guint                shrink : 1;
   guint                nrows;
@@ -207,8 +211,9 @@ static GParamSpec *plugin_props[N_PROPERTIES] = { NULL, };
 
 
 G_DEFINE_TYPE_WITH_CODE (XfcePanelPlugin, xfce_panel_plugin, GTK_TYPE_EVENT_BOX,
+                         G_ADD_PRIVATE (XfcePanelPlugin)
                          G_IMPLEMENT_INTERFACE (XFCE_TYPE_PANEL_PLUGIN_PROVIDER,
-                         xfce_panel_plugin_provider_init));
+                                                xfce_panel_plugin_provider_init));
 
 
 
@@ -218,8 +223,6 @@ xfce_panel_plugin_class_init (XfcePanelPluginClass *klass)
   GObjectClass   *gobject_class;
   GtkWidgetClass *gtkwidget_class;
 
-  g_type_class_add_private (klass, sizeof (XfcePanelPluginPrivate));
-
   klass->construct = NULL;
 
   gobject_class = G_OBJECT_CLASS (klass);
@@ -573,6 +576,22 @@ xfce_panel_plugin_class_init (XfcePanelPluginClass *klass)
                         G_PARAM_READABLE
                         | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * XfcePanelPlugin:icon-size:
+   *
+   * The icon-size in pixels of the #XfcePanelPlugin. Plugin writers can use it to read the
+   * plugin's icon size, but xfce_panel_plugin_get_icon_size() is recommended.
+   *
+   * Since: 4.14
+   **/
+  plugin_props[PROP_ICON_SIZE] =
+      g_param_spec_int ("icon-size",
+                        "Icon Size",
+                        "Size of the plugin's icon",
+                        0, (256 * 6), 0,
+                        G_PARAM_READABLE
+                        | G_PARAM_STATIC_STRINGS);
+
   /**
    * XfcePanelPlugin:screen-position:
    *
@@ -681,7 +700,7 @@ xfce_panel_plugin_class_init (XfcePanelPluginClass *klass)
 static void
 xfce_panel_plugin_init (XfcePanelPlugin *plugin)
 {
-  plugin->priv = G_TYPE_INSTANCE_GET_PRIVATE (plugin, XFCE_TYPE_PANEL_PLUGIN, XfcePanelPluginPrivate);
+  plugin->priv = xfce_panel_plugin_get_instance_private (plugin);
 
   plugin->priv->name = NULL;
   plugin->priv->display_name = NULL;
@@ -690,6 +709,7 @@ xfce_panel_plugin_init (XfcePanelPlugin *plugin)
   plugin->priv->property_base = NULL;
   plugin->priv->arguments = NULL;
   plugin->priv->size = 0;
+  plugin->priv->icon_size = 0;
   plugin->priv->small = FALSE;
   plugin->priv->expand = FALSE;
   plugin->priv->shrink = FALSE;
@@ -722,6 +742,7 @@ xfce_panel_plugin_provider_init (XfcePanelPluginProviderInterface *iface)
   iface->get_name = (ProviderToPluginChar) xfce_panel_plugin_get_name;
   iface->get_unique_id = (ProviderToPluginInt) xfce_panel_plugin_get_unique_id;
   iface->set_size = xfce_panel_plugin_set_size;
+  iface->set_icon_size = xfce_panel_plugin_set_icon_size;
   iface->set_mode = xfce_panel_plugin_set_mode;
   iface->set_nrows = xfce_panel_plugin_set_nrows;
   iface->set_screen_position = xfce_panel_plugin_set_screen_position;
@@ -793,6 +814,10 @@ xfce_panel_plugin_get_property (GObject    *object,
       g_value_set_int (value, private->size * private->nrows);
       break;
 
+    case PROP_ICON_SIZE:
+      g_value_set_uint (value, private->icon_size);
+      break;
+
     case PROP_NROWS:
       g_value_set_uint (value, private->nrows);
       break;
@@ -915,6 +940,13 @@ xfce_panel_plugin_finalize (GObject *object)
   /* destroy the menu */
   if (plugin->priv->menu != NULL)
     {
+      /* remove custom items before they get destroyed */
+      for (li = plugin->priv->menu_items; li != NULL; li = li->next)
+        {
+          gtk_container_remove (GTK_CONTAINER (plugin->priv->menu), GTK_WIDGET (li->data));
+          g_object_unref (G_OBJECT (li->data));
+        }
+      g_slist_free (plugin->priv->menu_items);
       /* attached menu is destroyed by GtkWidget */
       panel_assert (plugin->priv->menu_items == NULL);
     }
@@ -991,13 +1023,12 @@ xfce_panel_plugin_button_press_event (GtkWidget      *widget,
         gtk_widget_set_sensitive (item, plugin->priv->menu_blocked == 0);
 
       /* popup the menu */
-#if GTK_CHECK_VERSION (3, 22, 0)
+#if GTK_CHECK_VERSION (3, 0, 0)
       gtk_menu_popup_at_pointer (menu, (GdkEvent *) event);
 #else
       gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
                       event->button, event->time);
 #endif
-
       return TRUE;
     }
 
@@ -1444,6 +1475,24 @@ xfce_panel_plugin_set_size (XfcePanelPluginProvider *provider,
 
 
 
+static void
+xfce_panel_plugin_set_icon_size (XfcePanelPluginProvider *provider,
+                                 gint                     icon_size)
+{
+  XfcePanelPlugin *plugin = XFCE_PANEL_PLUGIN (provider);
+
+  if (G_LIKELY (plugin->priv->icon_size != icon_size))
+    {
+      plugin->priv->icon_size = icon_size;
+      g_object_notify_by_pspec (G_OBJECT (plugin), plugin_props[PROP_ICON_SIZE]);
+
+      /* also update the size so the icon gets re-rendered */
+      xfce_panel_plugin_set_size (provider, -1);
+    }
+}
+
+
+
 static void
 xfce_panel_plugin_set_mode (XfcePanelPluginProvider *provider,
                             XfcePanelPluginMode      mode)
@@ -2030,7 +2079,8 @@ xfce_panel_plugin_set_small (XfcePanelPlugin *plugin,
  * xfce_panel_plugin_get_icon_size:
  * @plugin : an #XfcePanelPlugin,
  *
- * Returns a preferred icon size.
+ * Returns either the icon size defined in the panel's settings or
+ * a preferred icon size.
  *
  * Since: 4.14
  **/
@@ -2038,18 +2088,30 @@ gint
 xfce_panel_plugin_get_icon_size (XfcePanelPlugin *plugin)
 {
   gint width;
-  width = xfce_panel_plugin_get_size (plugin) / xfce_panel_plugin_get_nrows (plugin);
 
-  /* Since symbolic icons are usually only provided in 16px we
-   * try to be clever and use size steps */
-  if (width <= 27)
-    return 16;
-  else if (width < 34)
-    return 24;
-  else if (width < 40)
-    return 32;
+  g_return_val_if_fail (XFCE_IS_PANEL_PLUGIN (plugin), FALSE);
+  g_return_val_if_fail (XFCE_PANEL_PLUGIN_CONSTRUCTED (plugin), FALSE);
+
+  /* 0 is handled as 'automatic sizing' */
+  if (plugin->priv->icon_size == 0)
+    {
+      width = xfce_panel_plugin_get_size (plugin) / xfce_panel_plugin_get_nrows (plugin);
+
+      /* Since symbolic icons are usually only provided in 16px we
+      * try to be clever and use size steps */
+      if (width <= 27)
+        return 16;
+      else if (width < 34)
+        return 24;
+      else if (width < 40)
+        return 32;
+      else
+        return width;
+    }
   else
-    return width;
+    {
+      return plugin->priv->icon_size;
+    }
 }
 
 
@@ -2430,8 +2492,13 @@ xfce_panel_plugin_arrow_type (XfcePanelPlugin *plugin)
 {
   XfceScreenPosition  screen_position;
   GdkScreen          *screen;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GdkDisplay         *display;
+  GdkMonitor         *monitor;
+#else
   gint                monitor_num;
-  GdkRectangle        monitor;
+#endif
+  GdkRectangle        geometry;
   gint                x, y;
   GdkWindow          *window;
 
@@ -2458,17 +2525,22 @@ xfce_panel_plugin_arrow_type (XfcePanelPlugin *plugin)
 
       /* get the monitor geometry */
       screen = gtk_widget_get_screen (GTK_WIDGET (plugin));
+#if GTK_CHECK_VERSION (3, 0, 0)
+      display = gdk_screen_get_display (screen);
+      monitor = gdk_display_get_monitor_at_window (display, window);
+      gdk_monitor_get_geometry (monitor, &geometry);
+#else
       monitor_num = gdk_screen_get_monitor_at_window (screen, window);
-      gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
-
+      gdk_screen_get_monitor_geometry (screen, monitor_num, &geometry);
+#endif
       /* get the plugin root origin */
       gdk_window_get_root_origin (window, &x, &y);
 
       /* detect arrow type */
       if (screen_position == XFCE_SCREEN_POSITION_FLOATING_H)
-        return (y < (monitor.y + monitor.height / 2)) ? GTK_ARROW_DOWN : GTK_ARROW_UP;
+        return (y < (geometry.y + geometry.height / 2)) ? GTK_ARROW_DOWN : GTK_ARROW_UP;
       else
-        return (x < (monitor.x + monitor.width / 2)) ? GTK_ARROW_RIGHT : GTK_ARROW_LEFT;
+        return (x < (geometry.x + geometry.width / 2)) ? GTK_ARROW_RIGHT : GTK_ARROW_LEFT;
     }
 }
 
@@ -2501,8 +2573,13 @@ xfce_panel_plugin_position_widget (XfcePanelPlugin *plugin,
 {
   GtkRequisition  requisition;
   GdkScreen      *screen;
-  GdkRectangle    monitor;
+  GdkRectangle    geometry;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GdkDisplay     *display;
+  GdkMonitor     *monitor;
+#else
   gint            monitor_num;
+#endif
   GTimeVal        now_t, end_t;
   GtkWidget      *toplevel, *plug;
   gint            px, py;
@@ -2602,18 +2679,24 @@ xfce_panel_plugin_position_widget (XfcePanelPlugin *plugin,
 
   /* get the monitor geometry */
   screen = gtk_widget_get_screen (attach_widget);
+#if GTK_CHECK_VERSION (3, 0, 0)
+  display = gdk_screen_get_display (screen);
+  monitor = gdk_display_get_monitor_at_window (display, gtk_widget_get_window (attach_widget));
+  gdk_monitor_get_geometry (monitor, &geometry);
+#else
   monitor_num = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (attach_widget));
-  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+  gdk_screen_get_monitor_geometry (screen, monitor_num, &geometry);
+#endif
 
   /* keep the menu inside the screen */
-  if (*x > monitor.x + monitor.width - requisition.width)
-    *x = monitor.x + monitor.width - requisition.width;
-  if (*x < monitor.x)
-    *x = monitor.x;
-  if (*y > monitor.y + monitor.height - requisition.height)
-    *y = monitor.y + monitor.height - requisition.height;
-  if (*y < monitor.y)
-    *y = monitor.y;
+  if (*x > geometry.x + geometry.width - requisition.width)
+    *x = geometry.x + geometry.width - requisition.width;
+  if (*x < geometry.x)
+    *x = geometry.x;
+  if (*y > geometry.y + geometry.height - requisition.height)
+    *y = geometry.y + geometry.height - requisition.height;
+  if (*y < geometry.y)
+    *y = geometry.y;
 
   /* popup on the correct screen */
   if (G_LIKELY (GTK_IS_MENU (menu_widget)))
