@@ -300,6 +300,7 @@ panel_application_xfconf_window_bindings (PanelApplication *application,
     { "background-style", G_TYPE_UINT },
     { "background-rgba", GDK_TYPE_RGBA },
     { "background-image", G_TYPE_STRING },
+    { "icon-size", G_TYPE_UINT },
     { "output-name", G_TYPE_STRING },
     { "position", G_TYPE_STRING },
     { "disable-struts", G_TYPE_BOOLEAN },
@@ -393,7 +394,7 @@ panel_application_load_real (PanelApplication *application)
               && sscanf (output_name, "screen-%d", &screen_num) == 1)
             {
               if (screen_num < 1)
-                screen = gdk_display_get_screen (display, screen_num);
+                screen = gdk_display_get_default_screen (display);
             }
           g_free (output_name);
 
@@ -1479,7 +1480,7 @@ panel_application_new_window (PanelApplication *application,
   GtkWidget          *itembar;
   gchar              *property;
   gint                idx;
-  static const gchar *props[] = { "mode", "size", "nrows" };
+  static const gchar *props[] = { "mode", "size", "nrows", "icon-size" };
   guint               i;
   gchar              *position;
   static gint         unqiue_id_counter = 1;
@@ -1552,6 +1553,7 @@ panel_application_new_window (PanelApplication *application,
       idx = g_slist_index (application->windows, window);
       position = g_strdup_printf ("p=0;x=100;y=%d", 30 + (idx * (48 + 10)));
       g_object_set (G_OBJECT (window), "position", position, NULL);
+      g_object_set (G_OBJECT (window), "size", 48, NULL);
       g_free (position);
     }
 
