@@ -58,7 +58,8 @@ static gboolean panel_base_window_enter_notify_event          (GtkWidget
                                                                GdkEventCrossing     *event);
 static gboolean panel_base_window_leave_notify_event          (GtkWidget            *widget,
                                                                GdkEventCrossing     *event);
-static void     panel_base_window_composited_changed          (GtkWidget            *widget);
+static void     panel_base_window_composited_changed          (GdkScreen            *screen,
+                                                               GtkWidget            *widget);
 static gboolean panel_base_window_active_timeout              (gpointer              user_data);
 static void     panel_base_window_active_timeout_destroyed    (gpointer              user_data);
 static void     panel_base_window_set_background_color_css    (PanelBaseWindow      *window);
@@ -107,7 +108,7 @@ struct _PanelBaseWindowPrivate
 
 
 
-G_DEFINE_TYPE (PanelBaseWindow, panel_base_window, GTK_TYPE_WINDOW)
+G_DEFINE_TYPE_WITH_PRIVATE (PanelBaseWindow, panel_base_window, GTK_TYPE_WINDOW)
 
 
 
@@ -117,9 +118,6 @@ panel_base_window_class_init (PanelBaseWindowClass *klass)
   GObjectClass   *gobject_class;
   GtkWidgetClass *gtkwidget_class;
 
-  /* add private data */
-  g_type_class_add_private (klass, sizeof (PanelBaseWindowPrivate));
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->get_property = panel_base_window_get_property;
   gobject_class->set_property = panel_base_window_set_property;
@@ -128,7 +126,6 @@ panel_base_window_class_init (PanelBaseWindowClass *klass)
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->enter_notify_event = panel_base_window_enter_notify_event;
   gtkwidget_class->leave_notify_event = panel_base_window_leave_notify_event;
-  gtkwidget_class->composited_changed = panel_base_window_composited_changed;
   gtkwidget_class->screen_changed = panel_base_window_screen_changed;
 
   g_object_class_install_property (gobject_class,
@@ -197,8 +194,9 @@ static void
 panel_base_window_init (PanelBaseWindow *window)
 {
   GtkStyleContext *context;
+  GdkScreen       *screen;
 
-  window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window, PANEL_TYPE_BASE_WINDOW, PanelBaseWindowPrivate);
+  window->priv = panel_base_window_get_instance_private (window);
 
   window->is_composited = FALSE;
   window->background_style = PANEL_BG_STYLE_NONE;
@@ -206,11 +204,16 @@ panel_base_window_init (PanelBaseWindow *window)
   window->background_rgba = NULL;
   window->enter_opacity = 1.00;
   window->leave_opacity = 1.00;
+  window->leave_opacity_old = 1.00;
 
   window->priv->css_provider = gtk_css_provider_new ();
   window->priv->borders = PANEL_BORDER_NONE;
   window->priv->active_timeout_id = 0;
 
+  screen = gtk_widget_get_screen (GTK_WIDGET (window));
+  g_signal_connect (G_OBJECT (screen), "composited-changed",
+                    G_CALLBACK (panel_base_window_composited_changed), window);
+
   /* some wm require stick to show the window on all workspaces, on xfwm4
    * the type-hint already takes care of that */
   gtk_window_stick (GTK_WINDOW (window));
@@ -448,11 +451,7 @@ panel_base_window_screen_changed (GtkWidget *widget, GdkScreen *previous_screen)
   if (visual != NULL)
     {
       gtk_widget_set_visual (widget, visual);
-#if GTK_CHECK_VERSION (3, 22, 0)
       window->is_composited = gdk_screen_is_composited (screen);
-#else
-      window->is_composited = gtk_widget_is_composited (widget);
-#endif
     }
 
    panel_debug (PANEL_DEBUG_BASE_WINDOW,
@@ -507,7 +506,8 @@ panel_base_window_leave_notify_event (GtkWidget        *widget,
 
 
 static void
-panel_base_window_composited_changed (GtkWidget *widget)
+panel_base_window_composited_changed (GdkScreen *screen,
+                                      GtkWidget *widget)
 {
   PanelBaseWindow *window = PANEL_BASE_WINDOW (widget);
   gboolean         was_composited = window->is_composited;
@@ -515,25 +515,30 @@ panel_base_window_composited_changed (GtkWidget *widget)
   GtkAllocation    allocation;
 
   /* set new compositing state */
-#if GTK_CHECK_VERSION (3, 22, 0)
-  GdkScreen       *screen;
-
-  screen = gtk_window_get_screen (GTK_WINDOW (window));
   window->is_composited = gdk_screen_is_composited (screen);
-#else
-  window->is_composited = gtk_widget_is_composited (widget);
-#endif
+
   if (window->is_composited == was_composited)
     return;
 
   if (window->is_composited)
     {
+      if (window->leave_opacity != window->leave_opacity_old)
+        window->leave_opacity = window->leave_opacity_old;
       gtk_widget_set_opacity (GTK_WIDGET (widget), window->leave_opacity);
       panel_base_window_set_plugin_data (window,
                                          panel_base_window_set_plugin_leave_opacity);
 
     }
-
+  else
+    {
+      /* make sure that the leave opacity is always disabled without compositing, but
+         remember the original value so we can reset it if compositing gets re-enabled */
+      window->leave_opacity_old = window->leave_opacity;
+      window->leave_opacity = 1.0;
+      gtk_widget_set_opacity (GTK_WIDGET (widget), window->leave_opacity);
+      panel_base_window_set_plugin_data (window,
+                                         panel_base_window_set_plugin_leave_opacity);
+    }
   panel_debug (PANEL_DEBUG_BASE_WINDOW,
                "%p: compositing=%s", window,
                PANEL_DEBUG_BOOL (window->is_composited));
@@ -580,10 +585,10 @@ panel_base_window_active_timeout (gpointer user_data)
 static void
 panel_base_window_active_timeout_destroyed (gpointer user_data)
 {
-  PANEL_BASE_WINDOW (user_data)->priv->active_timeout_id = 0;
   PanelBaseWindow        *window = PANEL_BASE_WINDOW (user_data);
   GtkStyleContext        *context;
 
+  window->priv->active_timeout_id = 0;
   context = gtk_widget_get_style_context (GTK_WIDGET (window));
   /* Stop the marching ants */
   gtk_style_context_remove_class (context, "marching-ants");
@@ -673,6 +678,31 @@ panel_base_window_reset_background_css (PanelBaseWindow *window) {
 
 
 
+void
+panel_base_window_orientation_changed (PanelBaseWindow *window,
+                                       gint             mode)
+{
+  GtkStyleContext         *context = gtk_widget_get_style_context (GTK_WIDGET (window));
+
+  /* Reset all orientation-related style-classes */
+  if (gtk_style_context_has_class (context, "horizontal"))
+    gtk_style_context_remove_class (context, "horizontal");
+  if (gtk_style_context_has_class (context, "vertical"))
+    gtk_style_context_remove_class (context, "vertical");
+  if (gtk_style_context_has_class (context, "deskbar"))
+    gtk_style_context_remove_class (context, "deskbar");
+
+  /* Apply the appropriate style-class */
+  if (mode == 0)
+    gtk_style_context_add_class (context, "horizontal");
+  else if (mode == 1)
+    gtk_style_context_add_class (context, "vertical");
+  else if (mode == 2)
+    gtk_style_context_add_class (context, "deskbar");
+}
+
+
+
 static void
 panel_base_window_set_plugin_data (PanelBaseWindow *window,
                                    GtkCallback      func)
