@@ -84,6 +84,8 @@ static const gchar *panel_plugin_external_get_name                (XfcePanelPlug
 static gint         panel_plugin_external_get_unique_id           (XfcePanelPluginProvider          *provider);
 static void         panel_plugin_external_set_size                (XfcePanelPluginProvider          *provider,
                                                                    gint                              size);
+static void         panel_plugin_external_set_icon_size           (XfcePanelPluginProvider          *provider,
+                                                                   gint                              icon_size);
 static void         panel_plugin_external_set_mode                (XfcePanelPluginProvider          *provider,
                                                                    XfcePanelPluginMode               mode);
 static void         panel_plugin_external_set_nrows               (XfcePanelPluginProvider          *provider,
@@ -139,7 +141,9 @@ enum
 
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PanelPluginExternal, panel_plugin_external, GTK_TYPE_SOCKET,
-  G_IMPLEMENT_INTERFACE (XFCE_TYPE_PANEL_PLUGIN_PROVIDER, panel_plugin_external_provider_init))
+                                  G_ADD_PRIVATE (PanelPluginExternal)
+                                  G_IMPLEMENT_INTERFACE (XFCE_TYPE_PANEL_PLUGIN_PROVIDER,
+                                                         panel_plugin_external_provider_init))
 
 
 
@@ -150,8 +154,6 @@ panel_plugin_external_class_init (PanelPluginExternalClass *klass)
   GtkWidgetClass *gtkwidget_class;
   GtkSocketClass *gtksocket_class;
 
-  g_type_class_add_private (klass, sizeof (PanelPluginExternalPrivate));
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = panel_plugin_external_finalize;
   gobject_class->set_property = panel_plugin_external_set_property;
@@ -195,7 +197,7 @@ panel_plugin_external_class_init (PanelPluginExternalClass *klass)
 static void
 panel_plugin_external_init (PanelPluginExternal *external)
 {
-  external->priv = G_TYPE_INSTANCE_GET_PRIVATE (external, PANEL_TYPE_PLUGIN_EXTERNAL, PanelPluginExternalPrivate);
+  external->priv = panel_plugin_external_get_instance_private (external);
 
   external->module = NULL;
   external->show_configure = FALSE;
@@ -222,6 +224,7 @@ panel_plugin_external_provider_init (XfcePanelPluginProviderInterface *iface)
   iface->get_name = panel_plugin_external_get_name;
   iface->get_unique_id = panel_plugin_external_get_unique_id;
   iface->set_size = panel_plugin_external_set_size;
+  iface->set_icon_size = panel_plugin_external_set_icon_size;
   iface->set_mode = panel_plugin_external_set_mode;
   iface->set_nrows = panel_plugin_external_set_nrows;
   iface->set_screen_position = panel_plugin_external_set_screen_position;
@@ -496,14 +499,13 @@ static void
 panel_plugin_external_child_spawn_child_setup (gpointer data)
 {
   PanelPluginExternal *external = PANEL_PLUGIN_EXTERNAL (data);
-  GdkScreen           *screen;
-  gchar               *name;
+  GdkDisplay          *display;
+  const gchar         *name;
 
   /* this is what gdk_spawn_on_screen does */
-  screen = gtk_widget_get_screen (GTK_WIDGET (external));
-  name = gdk_screen_make_display_name (screen);
+  display = gtk_widget_get_display (GTK_WIDGET (external));
+  name = gdk_display_get_name (display);
   g_setenv ("DISPLAY", name, TRUE);
-  g_free (name);
 }
 
 
@@ -900,6 +902,26 @@ panel_plugin_external_set_size (XfcePanelPluginProvider *provider,
 
 
 
+static void
+panel_plugin_external_set_icon_size (XfcePanelPluginProvider *provider,
+                                     gint                     icon_size)
+{
+  GValue value = { 0, };
+
+  panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (provider));
+  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
+
+  g_value_init (&value, G_TYPE_INT);
+  g_value_set_int (&value, icon_size);
+
+  panel_plugin_external_queue_add (PANEL_PLUGIN_EXTERNAL (provider),
+                                   PROVIDER_PROP_TYPE_SET_ICON_SIZE, &value);
+
+  g_value_unset (&value);
+}
+
+
+
 static void
 panel_plugin_external_set_mode (XfcePanelPluginProvider *provider,
                                 XfcePanelPluginMode      mode)
