@@ -73,6 +73,13 @@ static void                     panel_preferences_dialog_panel_add
                                                                                  PanelPreferencesDialog *dialog);
 static void                     panel_preferences_dialog_panel_remove           (GtkWidget              *widget,
                                                                                  PanelPreferencesDialog *dialog);
+static gboolean                 panel_preferences_dialog_icon_size_state_set    (GtkSwitch              *widget,
+                                                                                 gboolean                state,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_icon_size_changed      (GtkSpinButton          *button,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_compositing_clicked    (GtkButton              *button,
+                                                                                 gpointer                user_data);
 static void                     panel_preferences_dialog_panel_switch           (GtkWidget              *widget,
                                                                                  PanelPreferencesDialog *dialog);
 static XfcePanelPluginProvider *panel_preferences_dialog_item_get_selected      (PanelPreferencesDialog *dialog,
@@ -180,7 +187,9 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   GtkTreeViewColumn *column;
   GtkCellRenderer   *renderer;
   GtkTreeSelection  *selection;
-  gchar             *path;
+  gchar             *path_old;
+  gchar             *path_new;
+  gchar             *xfwm4_tweaks;
 
   dialog->bindings = NULL;
   dialog->application = panel_application_get ();
@@ -208,12 +217,12 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   connect_signal ("panel-remove", "clicked", panel_preferences_dialog_panel_remove);
   connect_signal ("panel-combobox", "changed", panel_preferences_dialog_panel_combobox_changed);
 
-  /* check if panel-switch is installed and if so show button */
+  /* check if xfce4-panel-profiles or panel-switch are installed and if either is show the button */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "panel-switch");
-  path = g_find_program_in_path ("xfpanel-switch");
-  if (path == NULL)
-    gtk_widget_set_visible (GTK_WIDGET (object), FALSE);
-
+  path_old = g_find_program_in_path ("xfpanel-switch");
+  path_new = g_find_program_in_path ("xfce4-panel-profiles");
+  if (path_new == NULL && path_old == NULL)
+    gtk_widget_hide (GTK_WIDGET (object));
   connect_signal ("panel-switch", "clicked", panel_preferences_dialog_panel_switch);
 
   /* style tab */
@@ -222,6 +231,16 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   g_signal_connect_swapped (G_OBJECT (object), "changed",
       G_CALLBACK (panel_preferences_dialog_bg_style_changed), dialog);
 
+  /* icon size switch, handling enabling/disabling of custom icon sizes */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-spinbutton");
+  panel_return_if_fail (G_IS_OBJECT (object));
+  g_signal_connect (G_OBJECT (object), "value-changed",
+                    G_CALLBACK (panel_preferences_dialog_icon_size_changed), dialog);
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-switch");
+  panel_return_if_fail (G_IS_OBJECT (object));
+  g_signal_connect (G_OBJECT (object), "state-set",
+                    G_CALLBACK (panel_preferences_dialog_icon_size_state_set), dialog);
+
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "composited");
   panel_return_if_fail (G_IS_OBJECT (object));
   g_signal_connect_swapped (G_OBJECT (object), "notify::visible",
@@ -233,6 +252,16 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
                           G_OBJECT (info), "visible",
                           G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
 
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "xfwm4-settings");
+  panel_return_if_fail (G_IS_OBJECT (object));
+  xfwm4_tweaks = g_find_program_in_path ("xfwm4-tweaks-settings");
+  if (xfwm4_tweaks != NULL)
+    gtk_widget_show (GTK_WIDGET (object));
+  else
+    gtk_widget_hide (GTK_WIDGET (object));
+  g_signal_connect (G_OBJECT (object), "clicked",
+                    G_CALLBACK (panel_preferences_dialog_compositing_clicked), xfwm4_tweaks);
+
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-image");
   panel_return_if_fail (GTK_IS_FILE_CHOOSER_BUTTON (object));
   g_signal_connect (G_OBJECT (object), "file-set",
@@ -275,7 +304,7 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   renderer = gtk_cell_renderer_pixbuf_new ();
   column = gtk_tree_view_column_new_with_attributes ("", renderer, "icon-name",
                                                      ITEM_COLUMN_ICON_NAME, NULL);
-  g_object_set (G_OBJECT (renderer), "stock-size", GTK_ICON_SIZE_BUTTON, NULL);
+  g_object_set (G_OBJECT (renderer), "stock-size", GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
   /* text renderer */
@@ -423,9 +452,9 @@ panel_preferences_dialog_bindings_add (PanelPreferencesDialog *dialog,
 static void
 panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
 {
-  GdkScreen   *screen;
   GdkDisplay  *display;
-  gint         n_screens, n_monitors = 1;
+  GdkMonitor  *monitor;
+  gint         n_monitors = 1;
   GObject     *object;
   GObject     *store;
   gchar       *output_name = NULL;
@@ -435,6 +464,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   gint         n = 0, i;
   gchar       *name, *title;
   gboolean     span_monitors_sensitive = FALSE;
+  gboolean     icon_size_set;
 
   /* leave when there is no active panel */
   panel_return_if_fail (G_IS_OBJECT (dialog->active));
@@ -456,6 +486,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   panel_preferences_dialog_bindings_add (dialog, "composited", "sensitive", G_BINDING_SYNC_CREATE);
   panel_preferences_dialog_bindings_add (dialog, "background-style", "active", 0);
   panel_preferences_dialog_bindings_add (dialog, "background-rgba", "rgba", 0);
+  panel_preferences_dialog_bindings_add (dialog, "icon-size", "value", 0);
 
   /* watch image changes from the panel */
   dialog->bg_image_notify_handler_id = g_signal_connect_swapped (G_OBJECT (dialog->active),
@@ -463,15 +494,9 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   panel_preferences_dialog_bg_image_notified (dialog);
 
 
-  /* get run mode of the driver (multiple screens or randr) */
-  screen = gtk_widget_get_screen (GTK_WIDGET (dialog->active));
+  /* get run mode of the driver (multiple monitors or randr) */
   display = gtk_widget_get_display (GTK_WIDGET (dialog->active));
-  n_screens = 1;
-  n_monitors = 1;
-  if (G_LIKELY (n_screens <= 1))
-    {
-      n_monitors = gdk_screen_get_n_monitors (screen);
-    }
+  n_monitors = gdk_display_get_n_monitors (display);
 
   /* update the output selector */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "output-name");
@@ -485,9 +510,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
 
   g_object_get (G_OBJECT (dialog->active), "output-name", &output_name, NULL);
 
-  if (n_screens > 1
-      || n_monitors > 1
-      || !panel_str_is_empty (output_name))
+  if (n_monitors > 1 || !panel_str_is_empty (output_name))
     {
       gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
                                          OUTPUT_NAME, "Automatic",
@@ -509,38 +532,12 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
           span_monitors_sensitive = TRUE;
         }
 
-      if (n_screens > 1)
-        {
-          for (i = 0; i < n_screens; i++)
-            {
-              /* warn the user about layouts we don't support */
-              screen = gdk_display_get_screen (display, i);
-              if (gdk_screen_get_n_monitors (screen) > 1)
-                g_message ("Screen %d has multiple monitors, the panel does not "
-                           "support such a configuration", i + 1);
-
-              /* I18N: screen name in the output selector */
-              title = g_strdup_printf (_("Screen %d"), i + 1);
-              name = g_strdup_printf ("screen-%d", i);
-              gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
-                                                 OUTPUT_NAME, name,
-                                                 OUTPUT_TITLE, title, -1);
-
-              if (!output_selected && g_strcmp0 (name, output_name) == 0)
-                {
-                  gtk_combo_box_set_active_iter  (GTK_COMBO_BOX (object), &iter);
-                  output_selected = TRUE;
-                }
-
-              g_free (name);
-              g_free (title);
-            }
-        }
-      else if (n_monitors >= 1)
+      if (n_monitors >= 1)
         {
           for (i = 0; i < n_monitors; i++)
             {
-              name = gdk_screen_get_monitor_plug_name (screen, i);
+              monitor = gdk_display_get_monitor(display, i);
+              name = g_strdup(gdk_monitor_get_model (monitor));
               if (panel_str_is_empty (name))
                 {
                   g_free (name);
@@ -589,7 +586,11 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   g_signal_handler_unblock (G_OBJECT (object), dialog->output_changed_handler_id);
 
   /* update visibility of the output selector */
-  object = gtk_builder_get_object (GTK_BUILDER (dialog), "output-box");
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "output-label");
+  panel_return_if_fail (GTK_IS_WIDGET (object));
+  g_object_set (G_OBJECT (object), "visible", selector_visible, NULL);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "output-name");
   panel_return_if_fail (GTK_IS_WIDGET (object));
   g_object_set (G_OBJECT (object), "visible", selector_visible, NULL);
 
@@ -605,6 +606,23 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "autohide-behavior");
   panel_return_if_fail (GTK_IS_COMBO_BOX (object));
   panel_preferences_dialog_autohide_changed (GTK_COMBO_BOX (object), dialog);
+
+  /* update visibility of the "icon-size" widgets */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-spinbutton");
+  panel_return_if_fail (G_IS_OBJECT (object));
+
+  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (object)) == 0)
+    icon_size_set = FALSE;
+  else
+    icon_size_set = TRUE;
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-revealer");
+  panel_return_if_fail (G_IS_OBJECT (object));
+  gtk_revealer_set_reveal_child (GTK_REVEALER (object), icon_size_set);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-switch");
+  panel_return_if_fail (G_IS_OBJECT (object));
+  gtk_switch_set_state (GTK_SWITCH (object), !icon_size_set);
 }
 
 
@@ -680,11 +698,19 @@ panel_preferences_dialog_bg_style_changed (PanelPreferencesDialog *dialog)
   g_object_get (G_OBJECT (dialog->active), "composited", &composited, NULL);
   gtk_color_chooser_set_use_alpha (GTK_COLOR_CHOOSER (object), composited);
 
-  object = gtk_builder_get_object (GTK_BUILDER (dialog), "bg-color-box");
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-rgba-label");
   panel_return_if_fail (GTK_IS_WIDGET (object));
   g_object_set (G_OBJECT (object), "visible", active == 1, NULL);
 
-  object = gtk_builder_get_object (GTK_BUILDER (dialog), "bg-image-box");
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-rgba");
+  panel_return_if_fail (GTK_IS_WIDGET (object));
+  g_object_set (G_OBJECT (object), "visible", active == 1, NULL);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-image-label");
+  panel_return_if_fail (GTK_IS_WIDGET (object));
+  g_object_set (G_OBJECT (object), "visible", active == 2, NULL);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-image");
   panel_return_if_fail (GTK_IS_WIDGET (object));
   g_object_set (G_OBJECT (object), "visible", active == 2, NULL);
 }
@@ -952,23 +978,103 @@ panel_preferences_dialog_panel_remove (GtkWidget              *widget,
 
 
 
+static gboolean
+panel_preferences_dialog_icon_size_state_set (GtkSwitch              *widget,
+                                              gboolean                state,
+                                              PanelPreferencesDialog *dialog)
+{
+  GObject   *spinbutton, *revealer;
+
+  panel_return_val_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog), FALSE);
+  spinbutton = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-spinbutton");
+  panel_return_val_if_fail (G_IS_OBJECT (spinbutton), FALSE);
+  revealer = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-revealer");
+  panel_return_val_if_fail (G_IS_OBJECT (revealer), FALSE);
+
+  /* we set the icon-size to 0 for auto-handling */
+  if (state)
+    {
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton), 0.0);
+    }
+  else
+    {
+      /* if the setting is initially enabled we start at the low default of 16px */
+      if (gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton)) == 0)
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton), 16.0);
+    }
+
+  gtk_revealer_set_reveal_child (GTK_REVEALER (revealer), !state);
+  gtk_switch_set_state (GTK_SWITCH (widget), state);
+
+  return TRUE;
+}
+
+
+
+static void
+panel_preferences_dialog_icon_size_changed (GtkSpinButton          *button,
+                                            PanelPreferencesDialog *dialog)
+{
+  GObject   *object;
+
+  panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "icon-size-switch");
+  panel_return_if_fail (G_IS_OBJECT (object));
+
+  /* 0 means the setting is disabled (icon-size-switch's state == true) so the lower value
+     of the spinbutton has to be 0. however, we want to set 16px as the reasonable lower limit for icons. */
+  if (gtk_spin_button_get_value_as_int (button) < 16
+      && gtk_switch_get_state (GTK_SWITCH (object)))
+      gtk_spin_button_set_value (button, 16.0);
+}
+
+
+
+static void
+panel_preferences_dialog_compositing_clicked (GtkButton *button, gpointer user_data)
+{
+  gchar    *command = user_data;
+  GAppInfo *app_info = NULL;
+  GError   *error = NULL;
+
+  app_info = g_app_info_create_from_commandline (command, "Window Manager Tweaks", G_APP_INFO_CREATE_NONE, &error);
+  if (G_UNLIKELY (app_info == NULL)) {
+    g_warning ("Could not find application %s", error->message);
+    return;
+  }
+  if (error != NULL)
+    g_error_free (error);
+
+  if (!g_app_info_launch (app_info, NULL, NULL, &error))
+    g_warning ("Could not launch the application %s", error->message);
+  if (error != NULL)
+    g_error_free (error);
+}
+
+
+
 static void
 panel_preferences_dialog_panel_switch (GtkWidget *widget, PanelPreferencesDialog *dialog)
 {
   GtkWidget *toplevel;
-  gchar     *path;
+  gchar     *path_old;
+  gchar     *path_new;
   GError    *error = NULL;
 
-  path = g_find_program_in_path ("xfpanel-switch");
-  if (path == NULL)
+  path_old = g_find_program_in_path ("xfpanel-switch");
+  path_new = g_find_program_in_path ("xfce4-panel-profiles");
+  if (path_old == NULL && path_new == NULL)
     return;
 
   /* close the preferences dialog */
   toplevel = gtk_widget_get_toplevel (widget);
   panel_preferences_dialog_response (toplevel, 0, dialog);
 
-  /* run xfpanel-switch */
-  g_spawn_command_line_async (path, &error);
+  /* first try the new name of the executable, then the old */
+  if (path_new)
+    g_spawn_command_line_async (path_new, &error);
+  else if (path_old)
+    g_spawn_command_line_async (path_old, &error);
 }
 
 
