@@ -837,12 +837,12 @@ actions_plugin_action_dbus_xfsm (const gchar  *method,
         }
 
       g_object_unref (G_OBJECT (proxy));
-    }
 
-  if (retval)
-    {
-      g_variant_unref (retval);
-      return TRUE;
+      if (retval)
+        {
+          g_variant_unref (retval);
+          return TRUE;
+        }
     }
 
   return FALSE;
@@ -891,9 +891,17 @@ actions_plugin_actions_allowed (void)
   GError          *error = NULL;
 
   /* check for commands we use */
-  path = g_find_program_in_path ("gdmflexiserver");
+  path = g_find_program_in_path ("dm-tool");
   if (path != NULL)
     PANEL_SET_FLAG (allow_mask, ACTION_TYPE_SWITCH_USER);
+  else
+  {
+    /* check for gdmflexiserver if dm-tool is not present */
+    g_free (path);
+    path = g_find_program_in_path ("gdmflexiserver");
+    if (path != NULL)
+      PANEL_SET_FLAG (allow_mask, ACTION_TYPE_SWITCH_USER);
+  }
   g_free (path);
 
   path = g_find_program_in_path ("xflock4");
@@ -951,6 +959,7 @@ actions_plugin_action_activate (GtkWidget      *widget,
   gboolean       succeed = FALSE;
   XfconfChannel *channel;
   gboolean       allow_save;
+  gchar         *path;
 
   entry = g_object_get_qdata (G_OBJECT (widget), action_quark);
   panel_return_if_fail (entry != NULL);
@@ -1004,7 +1013,12 @@ actions_plugin_action_activate (GtkWidget      *widget,
       break;
 
     case ACTION_TYPE_SWITCH_USER:
-      succeed = g_spawn_command_line_async ("gdmflexiserver", &error);
+      path = g_find_program_in_path ("dm-tool");
+      if (path != NULL)
+        succeed = g_spawn_command_line_async ("dm-tool switch-to-greeter", &error);
+      else
+        succeed = g_spawn_command_line_async ("gdmflexiserver", &error);
+      g_free (path);
       break;
 
     case ACTION_TYPE_LOCK_SCREEN:
@@ -1329,16 +1343,9 @@ actions_plugin_menu (GtkWidget     *button,
         }
     }
 
-#if GTK_CHECK_VERSION (3, 22, 0)
   gtk_menu_popup_at_widget (GTK_MENU (plugin->menu), button,
                             xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)) == GTK_ORIENTATION_VERTICAL
-                            ? GDK_GRAVITY_WEST : GDK_GRAVITY_NORTH,
-                            xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)) == GTK_ORIENTATION_VERTICAL
-                            ? GDK_GRAVITY_EAST : GDK_GRAVITY_SOUTH,
+                            ? GDK_GRAVITY_NORTH_EAST : GDK_GRAVITY_SOUTH_WEST,
+                            GDK_GRAVITY_NORTH_WEST,
                             NULL);
-#else
-  gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
-                  button != NULL ? xfce_panel_plugin_position_menu : NULL,
-                  plugin, 1, gtk_get_current_event_time ());
-#endif
 }
