@@ -39,7 +39,7 @@
 #define DEFAULT_TITLE     _("Applications")
 #define DEFAULT_ICON_NAME "xfce4-panel-menu"
 #define DEFAULT_ICON_SIZE (16)
-
+#define DEFAULT_EDITOR    "menulibre"
 
 
 struct _ApplicationsMenuPluginClass
@@ -64,6 +64,7 @@ struct _ApplicationsMenuPlugin
   gchar           *button_icon;
   gboolean         custom_menu;
   gchar           *custom_menu_file;
+  gchar           *menu_editor;
 
   /* temp item we store here when the
    * properties dialog is opened */
@@ -83,7 +84,8 @@ enum
   PROP_BUTTON_TITLE,
   PROP_BUTTON_ICON,
   PROP_CUSTOM_MENU,
-  PROP_CUSTOM_MENU_FILE
+  PROP_CUSTOM_MENU_FILE,
+  PROP_MENU_EDITOR
 };
 
 
@@ -193,6 +195,13 @@ applications_menu_plugin_class_init (ApplicationsMenuPluginClass *klass)
                                                         NULL, NULL,
                                                         NULL,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_MENU_EDITOR,
+                                   g_param_spec_string ("menu-editor",
+                                                        NULL, NULL,
+                                                        DEFAULT_EDITOR,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 
@@ -286,6 +295,10 @@ applications_menu_plugin_get_property (GObject    *object,
       g_value_set_string (value, plugin->custom_menu_file);
       break;
 
+    case PROP_MENU_EDITOR:
+      g_value_set_string (value, plugin->menu_editor);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -371,6 +384,10 @@ applications_menu_plugin_set_property (GObject      *object,
         applications_menu_plugin_set_garcon_menu (plugin);
       break;
 
+    case PROP_MENU_EDITOR:
+      plugin->menu_editor = g_value_dup_string (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -399,6 +416,7 @@ applications_menu_plugin_construct (XfcePanelPlugin *panel_plugin)
     { "button-icon", G_TYPE_STRING },
     { "custom-menu", G_TYPE_BOOLEAN },
     { "custom-menu-file", G_TYPE_STRING },
+    { "menu-editor", G_TYPE_STRING },
     { NULL }
   };
 
@@ -412,6 +430,9 @@ applications_menu_plugin_construct (XfcePanelPlugin *panel_plugin)
   /* make sure the menu is set */
   applications_menu_plugin_set_garcon_menu (plugin);
 
+  if (!plugin->menu_editor)
+      plugin->menu_editor = DEFAULT_EDITOR;
+
   gtk_widget_show (plugin->button);
 
   applications_menu_plugin_size_changed (panel_plugin,
@@ -535,15 +556,10 @@ applications_menu_plugin_configure_plugin_icon_chooser (GtkWidget              *
 
   chooser = exo_icon_chooser_dialog_new (_("Select An Icon"),
                                          GTK_WINDOW (gtk_widget_get_toplevel (button)),
-                                         _("Cancel"), GTK_RESPONSE_CANCEL,
-                                         _("OK"), GTK_RESPONSE_ACCEPT,
+                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                         _("_OK"), GTK_RESPONSE_ACCEPT,
                                          NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (chooser), GTK_RESPONSE_ACCEPT);
-#if !GTK_CHECK_VERSION (3, 0, 0)
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (chooser),
-                                           GTK_RESPONSE_ACCEPT,
-                                           GTK_RESPONSE_CANCEL, -1);
-#endif
 
   exo_icon_chooser_dialog_set_icon (EXO_ICON_CHOOSER_DIALOG (chooser),
       panel_str_is_empty (plugin->button_icon) ? DEFAULT_ICON_NAME : plugin->button_icon);
@@ -570,15 +586,14 @@ applications_menu_plugin_configure_plugin_edit (GtkWidget              *button,
                                                 ApplicationsMenuPlugin *plugin)
 {
   GError      *error = NULL;
-  const gchar  command[] = "alacarte";
 
   panel_return_if_fail (XFCE_IS_APPLICATIONS_MENU_PLUGIN (plugin));
   panel_return_if_fail (GTK_IS_WIDGET (button));
 
-  if (!xfce_spawn_command_line_on_screen (gtk_widget_get_screen (button), command,
+  if (!xfce_spawn_command_line_on_screen (gtk_widget_get_screen (button), plugin->menu_editor,
                                           FALSE, FALSE, &error))
     {
-      xfce_dialog_show_error (NULL, error, _("Failed to execute command \"%s\"."), command);
+      xfce_dialog_show_error (NULL, error, _("Failed to execute command \"%s\"."), plugin->menu_editor);
       g_error_free (error);
     }
 }
@@ -633,7 +648,7 @@ applications_menu_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   /* whether we show the edit menu button */
   object = gtk_builder_get_object (builder, "edit-menu-button");
   panel_return_if_fail (GTK_IS_BUTTON (object));
-  path = g_find_program_in_path ("alacarte");
+  path = g_find_program_in_path (plugin->menu_editor);
   if (path != NULL)
     {
       object2 = gtk_builder_get_object (builder, "use-default-menu");
@@ -785,19 +800,11 @@ applications_menu_plugin_menu (GtkWidget              *button,
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
   /* show the menu */
-#if GTK_CHECK_VERSION (3, 22, 0)
   gtk_menu_popup_at_widget (GTK_MENU (plugin->menu), button,
                             xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)) == GTK_ORIENTATION_VERTICAL
-                            ? GDK_GRAVITY_WEST : GDK_GRAVITY_NORTH,
-                            xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)) == GTK_ORIENTATION_VERTICAL
-                            ? GDK_GRAVITY_EAST : GDK_GRAVITY_SOUTH,
+                            ? GDK_GRAVITY_NORTH_EAST : GDK_GRAVITY_SOUTH_WEST,
+                            GDK_GRAVITY_NORTH_WEST,
                             (GdkEvent *) event);
-#else
-  gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
-                  button != NULL ? xfce_panel_plugin_position_menu : NULL,
-                  plugin, 1,
-                  event != NULL ? event->time : gtk_get_current_event_time ());
-#endif
 
   return TRUE;
 }
