@@ -660,18 +660,10 @@ launcher_dialog_tree_popup_menu (GtkWidget            *treeview,
 
   /* show the menu */
   menu = gtk_builder_get_object (dialog->builder, "popup-menu");
-#if GTK_CHECK_VERSION (3, 22, 0)
   gtk_menu_popup_at_widget (GTK_MENU (menu), treeview,
                             GDK_GRAVITY_SOUTH_WEST,
                             GDK_GRAVITY_NORTH_WEST,
                             NULL);
-#else
-  gtk_menu_popup (GTK_MENU (menu),
-                  NULL, treeview,
-                  NULL, NULL, 3,
-                  gtk_get_current_event_time ());
-  #endif
-
   return TRUE;
 }
 
@@ -802,6 +794,15 @@ launcher_dialog_item_desktop_item_edit (GtkWidget            *widget,
 
 
 
+static void
+launcher_dialog_item_link_button_clicked (GtkWidget            *button,
+                                          LauncherPluginDialog *dialog)
+{
+  launcher_dialog_item_desktop_item_edit (button, "Link", NULL, dialog);
+}
+
+
+
 static void
 launcher_dialog_item_button_clicked (GtkWidget            *button,
                                      LauncherPluginDialog *dialog)
@@ -1200,6 +1201,9 @@ launcher_dialog_show (LauncherPlugin *plugin)
       g_signal_connect (G_OBJECT (object), "clicked",
           G_CALLBACK (launcher_dialog_item_button_clicked), dialog);
     }
+  object = gtk_builder_get_object (builder, "item-link");
+  g_signal_connect (G_OBJECT (object), "clicked",
+                    G_CALLBACK (launcher_dialog_item_link_button_clicked), dialog);
 
   /* connect menu items */
   for (i = 0; i < G_N_ELEMENTS (mi_names); i++)
