@@ -326,7 +326,10 @@ launcher_plugin_class_init (LauncherPluginClass *klass)
 static void
 launcher_plugin_init (LauncherPlugin *plugin)
 {
-  GtkIconTheme *icon_theme;
+  GtkIconTheme    *icon_theme;
+  GtkCssProvider  *css_provider;
+  GtkStyleContext *context;
+  gchar           *css_string;
 
   plugin->disable_tooltips = FALSE;
   plugin->move_first = FALSE;
@@ -372,6 +375,16 @@ launcher_plugin_init (LauncherPlugin *plugin)
   g_signal_connect_after (G_OBJECT (plugin->button), "draw",
       G_CALLBACK (launcher_plugin_button_draw), plugin);
 
+  /* Make sure there aren't any constraints set on buttons by themes (Adwaita sets those minimum sizes) */
+  context = gtk_widget_get_style_context (plugin->button);
+  css_provider = gtk_css_provider_new ();
+  css_string = g_strdup_printf ("#launcher-arrow { min-height: 0; min-width: 0; }");
+  gtk_css_provider_load_from_data (css_provider, css_string, -1, NULL);
+  gtk_style_context_add_provider (context,
+                                  GTK_STYLE_PROVIDER (css_provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_free (css_string);
+
   plugin->child = gtk_image_new ();
   gtk_container_add (GTK_CONTAINER (plugin->button), plugin->child);
 
@@ -779,7 +792,7 @@ launcher_plugin_items_load (LauncherPlugin *plugin,
               /* if something failed, use the pool item, but this one
                * won't be editable in the dialog */
               if (G_UNLIKELY (item == NULL))
-                item = g_object_ref (G_OBJECT (pool_item));
+                item = GARCON_MENU_ITEM (g_object_ref (G_OBJECT (pool_item)));
             }
 
           /* skip this item if still not found */
@@ -1262,15 +1275,14 @@ launcher_plugin_size_changed (XfcePanelPlugin *panel_plugin,
 {
   LauncherPlugin    *plugin = XFCE_LAUNCHER_PLUGIN (panel_plugin);
   gint               p_width, p_height;
-  gint               a_size;
-  gint               icon_size;
+  gint               a_width, a_height;
   gboolean           horizontal;
   LauncherArrowType  arrow_position;
 
   /* initialize the plugin size */
   size /= xfce_panel_plugin_get_nrows (panel_plugin);
   p_width = p_height = size;
-  icon_size = xfce_panel_plugin_get_icon_size (panel_plugin);
+  a_width = a_height = -1;
 
   /* add the arrow size */
   if (gtk_widget_get_visible (plugin->arrow))
@@ -1286,20 +1298,16 @@ launcher_plugin_size_changed (XfcePanelPlugin *panel_plugin,
         {
         case LAUNCHER_ARROW_NORTH:
         case LAUNCHER_ARROW_SOUTH:
+          a_height = ARROW_BUTTON_SIZE;
           if (!horizontal)
-            {
-              gtk_widget_get_preferred_height (plugin->arrow, NULL, &a_size);
-              p_height += a_size;
-            }
+            p_height += ARROW_BUTTON_SIZE;
           break;
 
         case LAUNCHER_ARROW_EAST:
         case LAUNCHER_ARROW_WEST:
+          a_width = ARROW_BUTTON_SIZE;
           if (horizontal)
-            {
-              gtk_widget_get_preferred_width (plugin->arrow, NULL, &a_size);
-              p_width += a_size;
-            }
+            p_width += ARROW_BUTTON_SIZE;
           break;
 
         default:
@@ -1307,13 +1315,22 @@ launcher_plugin_size_changed (XfcePanelPlugin *panel_plugin,
           panel_assert_not_reached ();
           break;
         }
+
+        /* set the arrow size */
+        gtk_widget_set_size_request (plugin->arrow, a_width, a_height);
+
     }
 
   /* set the panel plugin size */
-  if (plugin->show_label)
+  if (plugin->show_label) {
     gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), -1, -1);
+  }
   else {
+    gint             icon_size;
+
     gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), p_width, p_height);
+
+    icon_size = xfce_panel_plugin_get_icon_size (panel_plugin);
     /* if the icon is a pixbuf we have to recreate and scale it */
     if (plugin->pixbuf != NULL &&
         plugin->icon_name != NULL) {
@@ -1324,11 +1341,11 @@ launcher_plugin_size_changed (XfcePanelPlugin *panel_plugin,
       gtk_image_set_from_pixbuf (GTK_IMAGE (plugin->child), plugin->pixbuf);
     }
     /* set the panel plugin icon size */
-    else
-      gtk_image_set_pixel_size (GTK_IMAGE (plugin->child), icon_size);
+    else {
+      gtk_image_set_pixel_size (GTK_IMAGE (plugin->child), MIN (icon_size, icon_size));
+    }
   }
 
-
   return TRUE;
 }
 
@@ -1417,8 +1434,10 @@ launcher_plugin_pack_widgets (LauncherPlugin *plugin)
   panel_assert (pos != LAUNCHER_ARROW_DEFAULT);
 
   /* set the position of the arrow button in the box */
-  gtk_box_reorder_child (GTK_BOX (plugin->box), plugin->arrow,
-      (pos == LAUNCHER_ARROW_WEST || pos == LAUNCHER_ARROW_NORTH) ? 0 : -1);
+  gtk_box_set_child_packing (GTK_BOX (plugin->box), plugin->arrow, TRUE, TRUE, 0,
+                      (pos == LAUNCHER_ARROW_SOUTH || pos == LAUNCHER_ARROW_EAST) ? GTK_PACK_END : GTK_PACK_START);
+  gtk_box_set_child_packing (GTK_BOX (plugin->box), plugin->button, FALSE, FALSE, 0,
+                      (pos == LAUNCHER_ARROW_SOUTH || pos == LAUNCHER_ARROW_EAST) ? GTK_PACK_START : GTK_PACK_END);
 
   /* set the orientation */
   gtk_orientable_set_orientation (GTK_ORIENTABLE (plugin->box),
@@ -1449,7 +1468,9 @@ launcher_plugin_tooltip_pixbuf (GdkScreen   *screen,
   else
     theme = gtk_icon_theme_get_default ();
 
-  return gtk_icon_theme_load_icon (theme, icon_name, GTK_ICON_SIZE_DND, 0, NULL);
+  return gtk_icon_theme_load_icon_for_scale (theme, icon_name, GTK_ICON_SIZE_DND,
+                                             GTK_ICON_SIZE_DND,
+                                             GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
 }
 
 
@@ -1676,20 +1697,12 @@ launcher_plugin_menu_popup (gpointer user_data)
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), TRUE);
 
   /* popup the menu */
-#if GTK_CHECK_VERSION (3, 22, 0)
   gtk_menu_popup_at_widget (GTK_MENU (plugin->menu),
                             plugin->button,
                             xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)) == GTK_ORIENTATION_VERTICAL
-                            ? GDK_GRAVITY_WEST : GDK_GRAVITY_NORTH,
-                            xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin)) == GTK_ORIENTATION_VERTICAL
-                            ? GDK_GRAVITY_EAST : GDK_GRAVITY_SOUTH,
+                            ? GDK_GRAVITY_NORTH_EAST : GDK_GRAVITY_SOUTH_WEST,
+                            GDK_GRAVITY_NORTH_WEST,
                             NULL);
-#else
-  gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
-                  xfce_panel_plugin_position_menu,
-                  XFCE_PANEL_PLUGIN (plugin), 1,
-                  gtk_get_current_event_time ());
-#endif
 
   /* fallback to manual positioning, this is used with
    * drag motion over the arrow button */
@@ -1993,7 +2006,9 @@ launcher_plugin_supported_drop (GdkDragContext *context,
   GdkModifierType  modifiers = 0;
 
   /* do not handle drops if control is pressed */
-  gdk_window_get_pointer (gtk_widget_get_window (widget), NULL, NULL, &modifiers);
+  gdk_window_get_device_position (gtk_widget_get_window (widget),
+                                  gdk_drag_context_get_device(context),
+                                  NULL, NULL, &modifiers);
   if (PANEL_HAS_FLAG (modifiers, GDK_CONTROL_MASK))
     return GDK_NONE;
 
@@ -2237,8 +2252,8 @@ launcher_plugin_arrow_drag_leave_timeout (gpointer user_data)
     return FALSE;
 
   /* get the pointer position */
-  gdk_display_get_pointer (gtk_widget_get_display (menu),
-                           NULL, &pointer_x, &pointer_y, NULL);
+  gdk_device_get_position (gdk_seat_get_pointer (gdk_display_get_default_seat (gtk_widget_get_display (menu))),
+                           NULL, &pointer_x, &pointer_y);
 
   /* get the menu position */
   gdk_window_get_root_origin (gtk_widget_get_window (menu), &menu_x, &menu_y);
