@@ -326,7 +326,9 @@ systray_manager_register (SystrayManager  *manager,
   gtk_widget_add_events (invisible, GDK_PROPERTY_CHANGE_MASK | GDK_STRUCTURE_MASK);
 
   /* get the screen number */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   screen_number = gdk_screen_get_number (screen);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
   /* create the selection atom name */
   selection_name = g_strdup_printf ("_NET_SYSTEM_TRAY_S%d", screen_number);
@@ -340,7 +342,7 @@ systray_manager_register (SystrayManager  *manager,
   display = gdk_screen_get_display (screen);
 
   /* set the invisible window and take a reference */
-  manager->invisible = g_object_ref (G_OBJECT (invisible));
+  manager->invisible = GTK_WIDGET (g_object_ref (G_OBJECT (invisible)));
 
   /* set the visial property for transparent tray icons */
   systray_manager_set_visual (manager);
@@ -766,14 +768,8 @@ systray_manager_set_visual (SystrayManager *manager)
   visual_atom = gdk_x11_get_xatom_by_name_for_display (display,
       "_NET_SYSTEM_TRAY_VISUAL");
 
-#if GTK_CHECK_VERSION (3, 22, 0)
   if (gdk_screen_is_composited (gtk_widget_get_screen (manager->invisible))
       && (gdk_screen_get_rgba_visual (screen) != NULL))
-#else
-  if (gtk_widget_is_composited (manager->invisible)
-      && gdk_screen_get_rgba_visual (screen) != NULL
-      && gdk_display_supports_composite (display))
-#endif
     {
       /* get the rgba visual */
       xvisual = GDK_VISUAL_XVISUAL (gdk_screen_get_rgba_visual (screen));
